const createAuthorizationError = require('./authorization-error').createAuthorizationError;

function authorize(header, username, password) {
  const token = new Buffer(`${username}:${password}`).toString('base64');
  const expectedHeader = `Basic ${token}`;
  return header === expectedHeader;
}

function authorizeRequest(req, res, next) {
  if (!req.headers.authorization ||
    !authorize(req.headers.authorization, process.env.API_USERNAME, process.env.API_PASSWORD)) {
    const err = createAuthorizationError();
    next(err);
    req.emit('request-unauthorized', { err, url: req.originalUrl });
  } else {
    next();
    req.emit('request-authorized', { url: req.originalUrl });
  }
}

module.exports.authorize = authorize;
module.exports.authorizeRequest = authorizeRequest;
