const authorizationError = require('../index').authorizationError;
const expect = require('chai').expect;
const sinon = require('sinon');

describe('authorization-error.js', () => {
  describe('createAuthorizationError', () => {
    it('it should return an Error object with an isAuthorizationError property set to true.', () => {
      const error = authorizationError.createAuthorizationError();
      expect(error.isAuthorizationError).to.eql(true);
    });
  });
  describe('handleAuthorizationError', () => {
    describe('if handleAuthorizationError is called as an error handler with all four parameters', () => {
      it('if the error parameter does not have an isAuthorizationError property it should call next with the original error.', (done) => {
        const error = new Error('Not an authorization error.');
        authorizationError.handleAuthorizationError(error, null, null, (err) => {
          expect(err).to.eql(error);
          done();
        });
      });

      it('if the error parameter has an isAuthorizationError property that is true it should respond with a http 401 and the message "Authorization Failed."', () => {
        const error = authorizationError.createAuthorizationError();
        const res = {
          status: sinon.stub(),
          json: sinon.spy(),
        };
        res.status.returns(res);

        authorizationError.handleAuthorizationError(error, null, res, () => { });

        sinon.assert.calledWith(res.status, 401);
        sinon.assert.calledWith(res.json, { message: 'Authorization Failed.' });
      });

      it('if the error parameter has an isAuthorizationError property that is true and a json property with the value { msg: "Overridden error text." } it should respond with a http 401 and the json { msg: "Overridden error text." }', () => {
        const error = authorizationError.createAuthorizationError();
        error.json = { msg: 'Overriden error text.' };

        const res = {
          status: sinon.stub(),
          json: sinon.spy(),
        };
        res.status.returns(res);

        authorizationError.handleAuthorizationError(error, null, res, () => { });

        sinon.assert.calledWith(res.status, 401);
        sinon.assert.calledWith(res.json, error.json);
      });

      it('if the error parameter has an isAuthorizationError property that is true it should not call next.', () => {
        const error = authorizationError.createAuthorizationError();
        const res = {
          status: sinon.stub(),
          json: sinon.spy(),
        };
        res.status.returns(res);

        const next = sinon.spy();
        authorizationError.handleAuthorizationError(error, null, res, next);
        sinon.assert.notCalled(next);
      });
    });
  });
});
