const EventEmitter = require('events');
const expect = require('chai').expect;
const sessionAuthorization = require('../index').sessionAuthorization;
const sinon = require('sinon');

describe('session-authorization.js', () => {
  describe('authenticate', () => {
    describe('if req.user is an object', () => {
      let req;
      beforeEach(() => {
        req = new EventEmitter();
        req.originalUrl = '/api/resource/1234567890';
        req.user = { _id: '1234567890' };
      });

      it('if req.user is an object it should call next.', () => {
        const next = sinon.spy();
        sessionAuthorization.authenticate(req, null, next);
        sinon.assert.calledOnce(next);
      });

      it('if req.user is an object it emit a "request-authorized" event.', (done) => {
        sessionAuthorization.authenticate(req, null, () => {
          req.on('request-authorized', (event) => {
            expect(event).to.eql({ user: req.user, url: req.originalUrl });
            done();
          });
        });
      });
    });

    describe('if req.user does not exist', () => {
      let req;
      beforeEach(() => {
        req = new EventEmitter();
        req.originalUrl = '/api/resource/1234567890';
      });

      it('it should call next with an authorization error.', (done) => {
        sessionAuthorization.authenticate(req, null, (err) => {
          expect(err.isAuthorizationError).to.eql(true);
          done();
        });
      });

      it('it should emit a "request-unauthorized" event.', (done) => {
        sessionAuthorization.authenticate(req, null, (err) => {
          req.on('request-unauthorized', (event) => {
            expect(event).to.eql({ err, url: req.originalUrl });
            done();
          });
        });
      });
    });
  });
});
